
//Remote Starter Hack Project
// Car Battery Voltage Hooked up to P15 (analog in)
//DS18B20 hooked up to P12 write IO
//Remote Start Output on P25
//read own source volatge with dual bond pins 11(adc in) and 27 (write high)
//Kevin Darrah 2015

//Note all code lives here in callbacks.c
#include "callbacks.h"

//global variables
int connected_status = 0;//0 when disconnected, 1 when connected
int tempC_decimal=0; //global so we can hold the temperature outside of the function that reads teh temperature, needed in case of CRC error, we can send old temperature
int remoteStarterCount = 0;//used when a remote start command is sent from device - counts down to 0

// DEFINES HERE
#define ledPin 0 // p0
#define tempPin 12 // p12
#define remoteStartPin 25 // p25
#define carBatteryPin 15 //p15
#define sourceVoltagePin 11 //p11
#define sourceVoltagePin_driver 27 //p27

//FUNCTIONS HERE
unsigned char DSB_byte_read(int pin);
void DSB_byte_write(int command, int pin);
char DSB_search_read(int pin);


// called at the interval time - set for 1000ms
void interval() {

	 char notifyString[32];//used for notifications back to device
	 unsigned char data_in[10];//used by DS18B20 read
	 int i=0;//for loop var
	 int j=0;//for loop var
	 unsigned char CRC=0;//CRC var's for DS18B20
	 unsigned char CRCxor, CRCXOR3, CRCXOR4;

	 remoteStarterCount--;//counts down to 0 - used to hold remote start button for 2 seconds
	 if(remoteStarterCount<=0){
		 remoteStarterCount=0;
		 AIR_GPIO_Write(remoteStartPin, 0);//drive remote start pin LOW when timeout is complete
	 }

	 //if we're connected, run all of this code:
	if(connected_status == 1){
		AIR_GPIO_Write(ledPin, 1);//drive the LED HIGH when this code runs

		//DS18B20 BIT BANG CODE HERE - note that this code  is all lifted from my arduino DS18B20 project:
		// https://youtu.be/ZKNQhzPwH0s

			//FIRST RESET
			AIR_GPIO_SetMode(tempPin,AIR_GPIO_OUTPUT_PUSH_PULL);
			AIR_GPIO_Write(tempPin, 0);
			bleapputils_delayUs(500);
			AIR_GPIO_SetMode(tempPin,AIR_GPIO_INPUT_HIGH_IMPEDANCE);
			bleapputils_delayUs(500);

			//SEND 0xCC SKIP ROM
			DSB_byte_write(0xCC, tempPin);

			//SEND 0x44 Start Conversion
			DSB_byte_write(0x44, tempPin);

			//Wait for conversion to complete
			  while(1){
			  if(DSB_search_read(tempPin)==1)
			  break;
			  }//while

			//RESET
			AIR_GPIO_SetMode(tempPin,AIR_GPIO_OUTPUT_PUSH_PULL);
			AIR_GPIO_Write(tempPin, 0);
			bleapputils_delayUs(500);
			AIR_GPIO_SetMode(tempPin,AIR_GPIO_INPUT_HIGH_IMPEDANCE);
			bleapputils_delayUs(500);

			//SKIP ROM
			DSB_byte_write(0xCC, tempPin);

			//READ SCRATCH
			DSB_byte_write(0xBE, tempPin);

			//Now pull in the 9 bytes from the scratch pad
			for(i=0; i<9; i++){//9 bytes are read in
			  data_in[i] = DSB_byte_read(tempPin);//keep it all in data_in[0..8]
			  }

			//Run the CRC check on the first 8 bytes
			 for(i=0; i<8; i++){//CRC checker, 8 bytes wide, 9th byte is the CRC
			   for(j=0; j<8; j++){//8 bits each
			   CRCxor = ((data_in[i]>>j) & 0x01) ^ (CRC & 0x01);//XOR CRC bit 0 with the incoming bit
			   CRCXOR3 = CRCxor ^ ((CRC>>3) & 0x01);
			   if(CRCXOR3 == 0)
				  CRC &= ~(1<<3);
			   else
				   CRC |= (1<<3);
			   CRCXOR4 = CRCxor ^ ((CRC>>4) & 0x01);
			   if(CRCXOR4 == 0)
				  CRC &= ~(1<<4);
			   else
				   CRC |= (1<<4);
			   CRC = CRC>>1;//shift the whole CRC byte over to the right once
			   if((CRCxor & 0x01) == 0)
					  CRC &= ~(1<<7);
				   else
					   CRC |= (1<<7);
			   }//this goes on for the entire 8 bytes
			  }//for i=0...

			 //used for debugging here F means a fault, G means a pass
			 unsigned char CRC_Fault = 'F';
			 if(CRC == data_in[8]){// we matched the CRC byte
				 CRC_Fault = 'G';
				 tempC_decimal = (data_in[0] + (data_in[1]<<8))<<18;//convert the 12 bit signed value into a 32 bit
				 tempC_decimal = tempC_decimal>>18;//should be 18 for 12bit, or 22 to shave off float
			 }
			 //not that if a CRC match is not met, the tempC_decimal value is not updated, thus the last good value is sent to the app

	//DS18B20 code is over, now get the other information for the notification
    int source_voltage = AIR_ADC_Read(sourceVoltagePin);//this is the voltage sourcing the module
    int carBatteryVolatge = AIR_ADC_Read(carBatteryPin);//this is the divided down (=V/11) voltage which would be connected to the car battery
	//this sprintf is pretty cool, we format the string we're going to send up to the device
    sprintf(notifyString, ",%i,%i,%i,\r\n", tempC_decimal,source_voltage,carBatteryVolatge);
    //now that we have the string put together, we can send it out, here's how that's done:
	ATMOSPHERE_NotifyFunction(ID_NOTIFY, notifyString);//we have to put in ID_ before our function name
	AIR_GPIO_Write(ledPin, 0);//all done, turn the LED OFF
	}//connection status
}//interval


void connected() {//called once when connected
	connected_status = 1;//change status
	//the source volatge pin here is driven high, because it is bonded to pin 11, which is configured
	//as an analog input, so when we wirte 27 high, pin 11 is read as the source voltage - it's written LOW when disconnected to conserve power.
	AIR_GPIO_Write(sourceVoltagePin_driver, 1);//write 27 high
}


void disconnected() {//called once when disconnected
	connected_status = 0;//change status back
	AIR_GPIO_Write(remoteStartPin, 0);//kill the remote starter, just in case we disconnect before the countdown is complete
	AIR_GPIO_Write(sourceVoltagePin_driver, 0);//write 27 low - for low power reasons
}


void setup() {//setup function - just like arduino, called on bootup
	//Initialization - not using everything, so just comment out what is not needed
	AIR_GPIO_Init();
	AIR_POWER_Init();
	//AIR_UART_Init();
	//AIR_I2C_Init();
	//AIR_SPI_Init();
	//BUZZER_Init();

    //AIR_POWER_SetMode(AIR_POWER_NOSLEEP);
	AIR_POWER_SetMode(AIR_POWER_SLEEP);//with this mode, we can get down ~30uA when idling
	//DRIVE ALL UNUSED PINS LOW FOR MAXIMUM LOW POWER PERFORMANCE
	AIR_GPIO_SetMode(15, AIR_GPIO_OUTPUT_PUSH_PULL);
	AIR_GPIO_Write(15, 0);
	AIR_GPIO_SetMode(3, AIR_GPIO_OUTPUT_PUSH_PULL);
	AIR_GPIO_Write(3, 0);
	AIR_GPIO_SetMode(2, AIR_GPIO_OUTPUT_PUSH_PULL);
	AIR_GPIO_Write(2, 0);
	AIR_GPIO_SetMode(4, AIR_GPIO_OUTPUT_PUSH_PULL);
	AIR_GPIO_Write(4, 0);
	AIR_GPIO_SetMode(25, AIR_GPIO_OUTPUT_PUSH_PULL);
	AIR_GPIO_Write(25, 0);
	AIR_GPIO_SetMode(24, AIR_GPIO_OUTPUT_PUSH_PULL);
	AIR_GPIO_Write(24, 0);
	AIR_GPIO_SetMode(14, AIR_GPIO_OUTPUT_PUSH_PULL);
	AIR_GPIO_Write(14, 0);
	AIR_GPIO_SetMode(13, AIR_GPIO_OUTPUT_PUSH_PULL);
	AIR_GPIO_Write(13, 0);
	AIR_GPIO_SetMode(32, AIR_GPIO_OUTPUT_PUSH_PULL);
	AIR_GPIO_Write(32, 0);
	AIR_GPIO_SetMode(33, AIR_GPIO_OUTPUT_PUSH_PULL);
	AIR_GPIO_Write(33, 0);

	//These pins are actually used by the project
	AIR_GPIO_SetMode(remoteStartPin, AIR_GPIO_OUTPUT_PUSH_PULL);
	AIR_GPIO_Write(remoteStartPin, 0);
	AIR_GPIO_SetMode(ledPin, AIR_GPIO_OUTPUT_PUSH_PULL);
	AIR_GPIO_Write(ledPin, 0);
	AIR_ADC_Enable(sourceVoltagePin);//pin 11 is bonded with 27
	AIR_ADC_Enable(carBatteryPin);//car battery voltage
	AIR_GPIO_SetMode(sourceVoltagePin_driver, AIR_GPIO_OUTPUT_PUSH_PULL);
	AIR_GPIO_Write(sourceVoltagePin_driver, 0);//write 27 high
	//DS18B20 pin will already be taken care of, but it is pulled up, so that is good enough
}//setup


int INT(int data)//INT function created in the web IDE, used by the device to trigger a remote start
{
	if(data == 123456789){//data is what is receieved, and we implement a poor man's security code before allowing a remote start
	AIR_GPIO_Write(remoteStartPin, 1);//drive remote start button HIGH
	remoteStarterCount = 2;//count set to 2, so the button should be held in for ~2 seconds
	AIR_GPIO_Write(ledPin, 1);//blink the LED a bit
	bleapputils_delayUs(1000);//here is how delays can be used
	AIR_GPIO_Write(ledPin, 0);
	}


	return 0;//return 0, but could be used to read back to the device

}

char *JSON(char *data, int length)//not used - but can be used for communicating text back and forth
{

	return NULL;
	
}

char *NOTIFY(char *data, int length)//this function doesn't appear to be used, but it is for sending notification data back to the device
{

	return NULL;
	
}


/*
 * Note all DS18B20 code leveraged from one of my Arduino projects
 * https://www.youtube.com/watch?v=ZKNQhzPwH0s
 * Code here: http://www.kevindarrah.com/wp-content/uploads/2014/04/DS18B20.txt
 */

void DSB_byte_write(int command, int pin){//    BYTE WRITE START START
int i;
  for(i=0; i<8; i++){//8 bits in a byte, do this 8 times, least significant bit
	  AIR_GPIO_SetMode(pin,AIR_GPIO_OUTPUT_PUSH_PULL);//get ready to write
	  AIR_GPIO_Write(pin, 0);//go LOW for the entire 60ish microseconds
	  bleapputils_delayUs(1);
   if(((command >> i) & 0x01)>0){//if the first bit is a '1'
     //might get rid of this, probably not needed
	   AIR_GPIO_SetMode(pin,AIR_GPIO_INPUT_HIGH_IMPEDANCE);//since this is a '1', change into an input, and let the pull-up resistor yank the line HIGH
	   bleapputils_delayUs(59);//chill out for 60micro secs, to let the DS18B20 get a read of the high signal
   }//true
   else{//nope it was a '0'
	   bleapputils_delayUs(59);//the DS18B20 will come in and read the low line as a 0 here
	   AIR_GPIO_SetMode(pin,AIR_GPIO_INPUT_HIGH_IMPEDANCE);//back to input, line gets yanked high, and ready for the next bit
   }//else

  }//for i=0
}////    BYTE WRITE END END

unsigned char DSB_byte_read(int pin){//                  BYTE READ START START
  int j;
 unsigned char byte_in= 0;

  //reads a byte from the DS18B20, this is not a 'void' function so it returns the byte read in

 for(j=0; j<8; j++){//8 bits in a byte
	 //bleapputils_delayUs(1);
	 AIR_GPIO_Write(pin, 0);//drive low to tell the DS18B20 we're ready
	 AIR_GPIO_SetMode(pin,AIR_GPIO_OUTPUT_PUSH_PULL);//get ready

    //used to have a delay of 1us here, turned out it was giving corrupt data
	// bleapputils_delayUs(5);
	 AIR_GPIO_SetMode(pin,AIR_GPIO_INPUT_HIGH_IMPEDANCE);// change to an input to allow line to yank HIGH
	 //bleapputils_delayUs(2);//The DS18B20 now has control over the line
   //if pulled low, then we read in a '0', if high, then the bit is a '1'
	// bleapputils_delayUs(5);
   if(AIR_GPIO_Read(pin))//grab the line status
   byte_in |= (1<<j);//bitSet(byte_in,i);//set it or...
   else
	   byte_in &= ~(1<<j);//bitClear(byte_in,i);//clear it

   bleapputils_delayUs(50);//wait out the rest of the timing window
 }//for

  return(byte_in);

}////    BYTE READ END END

char DSB_search_read(int pin){// SEARCH READ START
   //delayMicroseconds(1);
//used by the Search routine
	AIR_GPIO_Write(pin, 0);//go low
	AIR_GPIO_SetMode(pin,AIR_GPIO_OUTPUT_PUSH_PULL);//trigger a reading

	//bleapputils_delayUs(5);//might get rid of
	AIR_GPIO_SetMode(pin,AIR_GPIO_INPUT_HIGH_IMPEDANCE);//release the line and wait
	//bleapputils_delayUs(5);

   //now see if teh line is high or low, indicating bit value
	if(AIR_GPIO_Read(pin)){
		bleapputils_delayUs(50);
   return 1;
   }
   else{
	   bleapputils_delayUs(50);
   return 0;

   }
}// SEARCH READ END



